@@ -79,4 +79,4 @@ class MsgErrs(ConstantDict):
 
     URL_INVALID = "URL_INVALID::{0}"
     URL_TIMEOUT = "URL_TIMEOUT::{0}"
-    URL_UNFETCHABLE = "URL_UNFETCHABLE::{0}"
+    URL_CONN_ERROR = "URL_CONN_ERROR::{0}"
