@@ -103,7 +103,7 @@ GeoipSchema = build_schema({
 
 HttpstatusSchema = build_schema({
     "properties": {
-        F.HTTP_STATUS: {"type": "number"},
+        F.HTTP_STATUS: {"type": ["number", "null"]},
     }
 })
 
