@@ -1,14 +1,16 @@
 # -*- coding: utf-8 -*-
 
 import logging
-import colander
+import jsonschema
 
 from datetime import datetime
-from .fields import MsgFields as F
+from .fields import MsgFields as F, MsgErrs as MERR
 from .exceptions import ValidationError
 import schemas
 
 
+log = logging.getLogger(__name__)
+
 SEED_VERSION = "9.9.9"
 
 
@@ -51,8 +53,13 @@ class TaskResult(dict):
                 not isinstance(self.task_version, basestring)):
             raise ValueError("Must define a task version as a string "
                              "(e.g. '1.2.2.'")
-        elif not self.tmoid or not isinstance(self.tmoid, int):
-            raise ValueError("Target monitoring id (tmoid) must be an int")
+
+        try:
+            self.tmoid = str(int(self.tmoid))
+        except TypeError:
+            raise ValueError("Target monitoring id (tmoid) must be an int: "
+                             "got %s of type %s"
+                             % (self.tmoid, type(self.tmoid)))
 
         # start the timer
         self.time_start = iso_now()
@@ -84,25 +91,28 @@ class TaskResult(dict):
         return cls(task, task_version, tmoid, data, schema=schema)
 
     def _set_schema(self, supplied_schema):
+        """
+        Automatically select a JSON schema to validate this TaskResult.
+        This is either based on the task_name (e.g. if task_name is abc,
+        _set_schema will attempt to load AbcSchema, or load a schema manually
+        supplied).
+        """
         schema_cls = None
 
         if not self.task_name:
             raise ValueError("Must specify a task_name")
 
-        print ("Supplied schema: %s" % supplied_schema)
         if not supplied_schema:
-            print "No schema supplied, going auto"
+            log.debug("Task name: {0}".format(self.task_name))
+            log.debug("Task name title: {0}".format(self.task_name.title()))
             schema_name = "{0}Schema".format(self.task_name.title())
             try:
                 schema_cls = getattr(schemas, schema_name)
             except AttributeError:
                 raise ValueError("Couldn't find validation schema called {0}"
-                                   .format(schema_name))
+                                 .format(schema_name))
         else:
             schema_cls = supplied_schema
-        print (schema_cls)
-        if schemas._BaseTaskSchema not in schema_cls.__bases__:
-            raise ValueError("schema must be of type colander.Schema")
 
         return schema_cls
 
@@ -126,7 +136,7 @@ class TaskResult(dict):
         try:
             return self[key]
         except KeyError:
-            logging.debug(self)
+            log.debug(self)
             raise ValueError("{0}.{1} has no value for {2}".format(
                 self.task_name, self.tmoid, key))
 
@@ -159,6 +169,16 @@ class TaskResult(dict):
         except KeyError:
             return
 
+    @property
+    def msg(self):
+        """
+        Return diagnostic messages returned by a task
+        """
+        try:
+            return self[F.META][F.META_MSG]
+        except KeyError:
+            return
+
     @property
     def version(self):
         """
@@ -182,14 +202,26 @@ class TaskResult(dict):
         self[F.META][F.META_END_TIME] = self.time_end
         self[F.META][F.META_VERSION] = self.task_version
 
-        self.schema_validate()
+        try:
+            self.schema_validate()
+        except ValidationError as e:
+            # The original error may have caused schema non-validation.
+            # Leave original error if exists, otherwise, add schema err.
+            if not self[F.META][F.META_ERR]:
+                self[F.META][F.META_ERR] = MERR.RESULT_INVALID.format(e)
+
+        if self.err:
+            log.debug(self.err)
 
     def schema_validate(self):
         try:
-            self.schema().deserialize(self)
-        except colander.Invalid as e:
+            jsonschema.validate(self, self.schema)
+        except Exception as e:
+            log.debug(self)
             raise ValidationError(
-                "Couldn't validate schema: {0}".format(e))
+                "Couldn't validate under {0} schema: {1}"
+                .format(self, e)
+            )
 
 
 class TaskSeed(TaskResult):
